<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @since      0.0.1
 *
 * @package    Wtsr
 * @subpackage Wtsr/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Wtsr
 * @subpackage Wtsr/admin
 * @author     Tobias Conrad <tc@santegra.de>
 */
class Wtsr_Admin {

    /**
     * The ID of this plugin.
     *
     * @since    0.0.1
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    0.0.1
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    0.0.1
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct( $plugin_name, $version ) {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    /**
     * Wordpress init.
     *
     * @since    0.0.1
     */
    public function init() {
        self::check_version();

        add_action( 'wp2leads_transfer_user_created', array($this, 'user_transfered_to_kt'), 15, 5 );
        add_action( 'wp2leads_transfer_user_updated', array($this, 'user_transfered_to_kt'), 15, 5 );
    }

    public function add_plugin_screen_link($links)
    {
        $links[] = '<a href="' . esc_url(get_admin_url(null, 'admin.php?page=wp2leads-wtsr')) . '">'. __( "Start", 'wp2leads-wtsr' ) .'</a>';
        return $links;
    }

    public function display_post_states($post_states, $post) {
        $wtsr_all_reviews_page = get_option('wtsr_all_reviews_page', false);

        if( !empty($wtsr_all_reviews_page) && $post->ID == $wtsr_all_reviews_page ) {
            $post_states[] = __('Custom all products review page', 'more-better-reviews-for-woocommerce');
        }

        return $post_states;
    }

    public function rest_api_init() {
        $controller = new Wtsr_Rest;
        $controller->register_routes();
    }

    public function license_version_changed() {
        $license_version = Wtsr_License::get_license_version();

        if ('pro' === $license_version) {
            TSManager::remove_reviews_limit_counter();
        }
    }

    public function new_product_review($comment_ID, $comment_approved, $commentdata) {
        $wtsr_review_approved = get_option('wtsr_review_approved', 'yes');

        if (!empty($comment_approved) || 'no' === $wtsr_review_approved) {
            $comment = get_comment( $comment_ID );

            $email = $comment->comment_author_email;
            $post_id = $comment->comment_post_ID;

            TSManager::maybe_product_page_review_reviewed($post_id, $email);
        }
    }

    public function edit_product_review($comment_ID, $data) {
        $wtsr_review_approved = get_option('wtsr_review_approved', 'yes');
        $comment = get_comment( $comment_ID );

        if (!empty($comment) && (!empty($comment->comment_approved) || 'no' === $wtsr_review_approved)) {
            $email = $comment->comment_author_email;
            $post_id = $comment->comment_post_ID;

            TSManager::maybe_product_page_review_reviewed($post_id, $email);
        }
    }

    public function product_reviews() {
        $is_ajax = defined( 'DOING_AJAX' ) && DOING_AJAX;

        if (!$is_ajax) {
            $hour = 12;
            $today              = date('Ymd', strtotime($hour . ':00:00'));
            $today_product_reviews = get_option('wtsr_product_reviews_' . $today);

            if (false === $today_product_reviews) {
                global $wpdb;

                $api = new TSApi('restricted');
                $today_product_reviews = $api->get_product_reviews();

                if (!empty($today_product_reviews)) {
                    $need_check = true;
                    $previous_product_reviews = $wpdb->get_row( "SELECT option_value FROM $wpdb->options WHERE option_name LIKE '%wtsr_product_reviews_%'", ARRAY_A );

                    if (!empty($previous_product_reviews)) {
                        $wpdb->query( "DELETE FROM $wpdb->options WHERE option_name LIKE '%wtsr_product_reviews_%'" );
                        $previous_product_reviews = $previous_product_reviews['option_value'];

                        $is_equal = strcmp($today_product_reviews, $previous_product_reviews);

                        if (0 === $is_equal) {
                            $need_check = false;
                        }
                    }

                    update_option('wtsr_product_reviews_' . $today, $today_product_reviews, false);

                    if ($need_check) {
                        update_option('wtsr_product_reviews_to_check', 1);
                    }
                }

                TSManager::check_if_product_reviews_reviewed_bg();
            } else {
                if (get_option('wtsr_product_reviews_to_check')) {
                    TSManager::check_if_product_reviews_reviewed_bg();
                }
            }
        }
    }

    public function check_settings() {
        $is_ajax = defined( 'DOING_AJAX' ) && DOING_AJAX;

        if (!$is_ajax) {
            delete_transient('wtsr_settings_errors');
            $is_woocommerce_only_mode = TSManager::is_woocommerce_only_mode();
            $is_woocommerce_reviews_disabled_globally = TSManager::is_woocommerce_reviews_disabled_globally();

            $default_rating_links = ReviewsModel::get_default_rating_links();
            $wtsr_rating_links = get_option('wtsr_rating_links', $default_rating_links);

            $errors = array();

            foreach ($wtsr_rating_links as $star => $rating_link) {
                if ($is_woocommerce_only_mode && 'wtsr_ts_review_link' === $rating_link) $rating_link = 'wtsr_product_url';
                if ($is_woocommerce_reviews_disabled_globally && 'wtsr_product_url' === $rating_link) $rating_link = 'wtsr_custom_link';

                if ('wtsr_custom_link' === $rating_link) {
                    if (empty($wtsr_rating_links["custom_link"][$star])) {
                        $errors[] = __('You need to specify custom links values on <a href="?page=wp2leads-wtsr&tab=email">Email tab</a>', 'more-better-reviews-for-woocommerce');

                        break;
                    }
                }
            }

            if (!empty($errors)) {
                $error_msg = implode(' ', $errors);

                if (!empty($error_msg)) {
                    set_transient('wtsr_settings_errors', $error_msg);
                }
            }
        }
    }

    public function set_ts_credentials() {
        $is_ajax = defined( 'DOING_AJAX' ) && DOING_AJAX;

        if (!$is_ajax) {
            $is_ts_credential_set = get_transient('wtsr_ts_credential_set');

            if (!$is_ts_credential_set) {
                delete_transient('wtsr_check_ts_credentials_empty');
                delete_transient('wtsr_check_ts_credentials');
                delete_transient('wtsr_check_ts_domain_credentials');
                delete_transient('wtsr_ts_credential_empty_set');
                delete_transient('wtsr_ts_domain_credentials_set');
                delete_option('wtsr_woocommerce_only_mode_enabled');

                $check_ts_credentials_empty = TSManager::check_ts_credentials_empty();

                if ($check_ts_credentials_empty) {
                    set_transient('wtsr_check_ts_credentials_empty', 1);
                } else {
                    delete_transient('wtsr_check_ts_credentials_empty');

                    $check_ts_credentials = TSManager::check_ts_credentials();

                    if (!empty($check_ts_credentials)) {
                        set_transient('wtsr_check_ts_credentials', $check_ts_credentials);
                    } else {
                        delete_transient('wtsr_check_ts_credentials');

                        $check_ts_domain_credentials = TSManager::check_ts_domain_credentials();

                        if (!empty($check_ts_domain_credentials)) {
                            set_transient('wtsr_check_ts_domain_credentials', $check_ts_domain_credentials);
                        } else {
                            delete_transient('wtsr_check_ts_domain_credentials');
                        }
                    }
                }

                set_transient('wtsr_ts_credential_set', 1, Wtsr_License::$license_check_timeout);
            }
        }
    }

    public function set_reviews_limitation() {
        $is_ajax = defined( 'DOING_AJAX' ) && DOING_AJAX;

        if (!$is_ajax) {
            $is_limit_set = get_transient('wtsr_limit_set');

            if (!$is_limit_set) {
                $limit_set = TSManager::set_reviews_limitation();

                if ($limit_set) {
                    set_transient('wtsr_limit_set', 1, Wtsr_License::$license_check_timeout);
                }
            }

            TSManager::reset_reviews_limit_counter();
        }
    }

    public function set_license() {
        $is_ajax = defined( 'DOING_AJAX' ) && DOING_AJAX;

        if (!$is_ajax) {

            $activation_in_progress = get_transient('wtsr_activation_in_progress');

            if ($activation_in_progress) {
                $activation_in_progress_timeout = get_transient('wtsr_activation_in_progress_timeout');

                if (!$activation_in_progress_timeout) {
                    Wtsr_License::close_license();
                    delete_transient('wtsr_activation_in_progress');
                }

                return;
            }

            $is_license_set = get_transient('wtsr_license_set');

            if (!$is_license_set) {
                Wtsr_License::set_license();
                set_transient('wtsr_license_set', 1, Wtsr_License::$license_check_timeout);
            }
        }
    }

    public function woocommerce_order() {
        $selected_order_status = get_option('wtsr_order_status', 'wc-completed');
        $is_wizard = Wtsr_Wizard::is_wizard();

        if (!$is_wizard) {
            if ( 'wc-order-created' === $selected_order_status ) {
                add_action( 'woocommerce_checkout_order_processed', 'TSManager::get_ts_link_new_order', 10, 3 );
            } else {
                add_action('woocommerce_order_status_changed', 'TSManager::get_ts_link_order_status_changed', 50, 4);
            }
        }

        add_action('wtsr_order_review_generate_request', 'TSManager::get_ts_link_new_order', 10, 3);
    }

    public function add_admin_notices() {
        Wtsr_Notices::admin_notices();
    }

    public function new_review_item_created($review_id, $order_id) {}
    public function review_status_changed( $status, $id, $order_id ) {
        // error_log( 'Status changed to ' . $status . ' for review # ' . $id . ' with order # ' . $order_id );
    }

    public static function check_version() {
        $version = get_option( 'wtsr_version' );

        if ( empty($version) || version_compare( $version, WTSR_VERSION, '<' ) ) {
            self::install();
            do_action( 'wtsr_updated' );
        }
    }

    public static function update_version() {
        delete_option( 'wtsr_version' );
        add_option( 'wtsr_version', WTSR_VERSION );
    }

    public static function install() {
        TSManager::clean_transients();
        ReviewsModel::create_table();
        self::update_version();
    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since    0.0.1
     */
    public function enqueue_styles() {
//        wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/wtsr-admin.css', array(), $this->version, 'all' );
    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    0.0.1
     */
    public function enqueue_global_scripts() {
        wp_enqueue_script( $this->plugin_name . '-global', plugin_dir_url( __FILE__ ) . 'js/wtsr-admin-global.js', array( 'jquery' ), $this->version . time(), true );
    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since    0.0.1
     */
    public function enqueue_global_styles() {
        wp_enqueue_style( 'wp-color-picker' );
        wp_enqueue_style( $this->plugin_name . '-jquery-ui', plugin_dir_url( __FILE__ ) . 'css/jquery-ui.min.css', array(), $this->version . time(), 'all' );
        wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/wtsr-admin.css', array(), $this->version . time(), 'all' );
    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    0.0.1
     */
    public function enqueue_scripts() {
        if ( ! did_action( 'wp_enqueue_media' ) ) {
            wp_enqueue_media();
        }

        wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/wtsr-admin.js', array( 'jquery', 'jquery-ui-core', 'jquery-ui-datepicker', 'wp-color-picker' ), $this->version . time(), true );

    }

    public function user_transfered_to_kt($map_id, $email, $data, $tags, $detach_tags) {
        $map = MapsModel::get($map_id);

        if (empty($map)) {
            return false;
        }

        $map_mapping = unserialize($map->mapping);
        $is_map_wtsr = false;

        foreach ($map_mapping['selects_only'] as $map_column) {
            if (
                'wp2lwtsr_reviews.id' === trim($map_column) ||
                'wp2lwtsr_reviews.order_id' === trim($map_column) ||
                'wp2lwtsr_reviews.email' === trim($map_column) ||
                'wp2lwtsr_reviews.status' === trim($map_column) ||
                'wp2lwtsr_reviews.review_link' === trim($map_column) ||
                'wp2lwtsr_reviews.review_message' === trim($map_column) ||
                'wp2lwtsr_reviews.review_created' === trim($map_column) ||
                'wp2lwtsr_reviews.review_sent' === trim($map_column)
            ) {
                $is_map_wtsr = true;
                break;
            }
        }

        if (!$is_map_wtsr) {
            return false;
        }

        $review = ReviewsModel::get_last_by_email($email);

        if (empty($review) || 'ready' !== trim($review['status'])) {
            return false;
        }

        $transfered = gmdate( 'Y-m-d H:i:s' );

        $data = array(
            'id' => $review['id'],
            'status' => 'transferred',
            'review_sent' => $transfered,
        );

        $id = ReviewsModel::update($data);

        $order = wc_get_order( $review['order_id'] );
        $order_email = !empty($order_data['billing']['email']) ? $order_data['billing']['email'] : false;
        $order_items = TSManager::get_order_items($order);
        $user_products = get_option('wtsr_transferred_' . $email, array());

        foreach ($order_items as $order_item) {
            $user_products[] = $order_item['sku'];
        }

        update_option('wtsr_transferred_' . $email, $user_products, false);

        return $id;
    }

    public function new_review_woo_item_created($review_id, $order_id) {
        $wtsr_email_send_via = TSManager::get_email_send_via();

        if (empty($wtsr_email_send_via) || 'woocommerce' !== $wtsr_email_send_via) {
            return false;
        }

        $wtsr_email_send_via_woocommerce_delay = get_option('wtsr_email_send_via_woocommerce_delay', '');

        if (empty($wtsr_email_send_via_woocommerce_delay)) {
            $mailer = WC()->mailer();
            $email = $mailer->emails['Wtsr_WC_Email_Review_Request'];
            $email->trigger( $order_id );
        } else {
            $now = time();
            $delay_array = explode(':', $wtsr_email_send_via_woocommerce_delay);

            if (count($delay_array)) {
                $is_immediately = false;

                foreach ($delay_array as $index => $delay_days) {
                    if (empty($delay_days)) {
                        $is_immediately = true;
                        unset($delay_array[$index]);
                    }
                }

                if (!empty($delay_array)) {
                    Wtsr_Cron::add_woo_email_schedule($review_id);
                }

                if ($is_immediately) {
                    $mailer = WC()->mailer();
                    $email = $mailer->emails['Wtsr_WC_Email_Review_Request'];
                    $email->trigger( $order_id );
                }
            }
        }
    }

    public function woo_review_request_email_sent($order_id, $scheduled) {
        $review = ReviewsModel::get_by_order_id($order_id);

        if (empty($review)) {
            return;
        }

        $review = $review[0];

        if (empty($review) || ('ready' !== trim($review->status) && 'woo-sent' !== trim($review->status))) {
            return false;
        }

        $order = wc_get_order(  $order_id );
        $note = __( 'Review Request Sent to customer', 'more-better-reviews-for-woocommerce' );
        $order->add_order_note( $note );
        $transfered = gmdate( 'Y-m-d H:i:s' );

        $data = array(
            'id' => $review->id,
            'status' => 'woo-sent',
            'review_sent' => $transfered,
        );

        $id = ReviewsModel::update($data);
    }

    /**
     * Add menu item.
     *
     * @since    0.0.1
     */
    public function admin_menu() {
        add_menu_page( __('Better Reviews for WooCommerce', 'more-better-reviews-for-woocommerce'), __('WOO better Reviews', 'more-better-reviews-for-woocommerce'), 'administrator', 'wp2leads-wtsr', array($this, 'render_settings_page'), '', '85.2' );
    }

    /**
     * Render settings page.
     *
     * @since    0.0.1
     */
    public function render_settings_page() {
        $reviews = ReviewsModel::get_all();
        if (!empty($reviews)) {
            $default_tab = 'reviews';
        } else {
            $default_tab = 'generate';
        }
        $active_tab = isset( $_GET[ 'tab' ] ) ? sanitize_text_field( $_GET[ 'tab' ] ) : $default_tab;

        $locale = get_user_locale();
        $locale_short = explode('_', $locale)[0];

        $required_plugins_wp2leads = Wtsr_Required_Plugins::get_required_plugins_wp2leads();
        $is_wp2leads_installed = Wtsr_Required_Plugins::is_plugin_installed( $required_plugins_wp2leads['slug'] ) && Wtsr_Required_Plugins::is_plugin_active( $required_plugins_wp2leads['slug'] );
        $wtsr_email_send_via = TSManager::get_email_send_via();
        $wtsr_email_send_via_woocommerce_delay = get_option('wtsr_email_send_via_woocommerce_delay', '');

        if ('wizard' === $active_tab) {
            include(dirname(__FILE__) . '/partials/wtsr-admin-wizard.php');
        } else {
            include(dirname(__FILE__) . '/partials/wtsr-admin-display.php');
        }
    }

    // TODO: This method can be removed
    public function product_review_check() {
        return;

        $is_ajax = defined( 'DOING_AJAX' ) && DOING_AJAX;

        if (!$is_ajax && !is_admin()) {
            if (function_exists('is_product') && is_product()) {
                if (!empty($_GET['ts_review'])) {
                    $rating = !empty($_GET['ts_rating']) ? sanitize_text_field( $_GET['ts_rating'] ) : '';
                    $order_id = !empty($_GET['ts_order_id']) ? sanitize_text_field( $_GET['ts_order_id'] ) : '';

                    $transferred = ReviewsModel::get_not_reviewed('order_id');

                    if (!empty($transferred[$order_id])) {
                        $review = $transferred[$order_id];

                        $data = array(
                            'id' => $review['id'],
                            'status' => 'reviewed',
                            'review_sent' => gmdate( 'Y-m-d H:i:s', time() )
                        );

                        $id = ReviewsModel::update($data);

                        if ($id) {
                            do_action('wtsr_review_status_changed', 'reviewed', $review['id'], $review['order_id']);
                        }
                    }
                }
            }
        }
    }

    public function checkbox_for_ts_review(  ) {
        $selected_review_ask = get_option('wtsr_review_ask', 'no');

        if ('yes' === $selected_review_ask) {
            $review_ask_template_editor = get_option('wtsr_review_ask_template_editor', TSManager::get_default_review_ask_template_editor());
            ?>
            <style>
                #ts_review_checkbox_field .woocommerce-form__label-for-checkbox {
                    position: relative;
                    padding-left: 25px;
                    display: block;
                }
                #ts_review_checkbox_field .woocommerce-form__label-for-checkbox .input-checkbox {
                    position: absolute;
                    left: 0;
                    top: 5px;
                }
                #ts_review_checkbox_field .woocommerce-form__label-for-checkbox .ts_review_checkbox-checkbox-text p {
                    margin-top: 0;
                    line-height: 1.2;
                }
            </style>
            <div id="ts_review_checkbox_field">
                <div class="form-row form-row-wide">
                    <label class="woocommerce-form__label woocommerce-form__label-for-checkbox checkbox">
                        <input type="checkbox" class="woocommerce-form__input woocommerce-form__input-checkbox input-checkbox" name="ts_review_checkbox" id="ts_review_checkbox" />
                        <div class="ts_review_checkbox-checkbox-text"><?php echo wpautop($review_ask_template_editor); ?></div>
                    </label>
                </div>
            </div>
            <?php
        }
    }

    public function checkbox_for_ts_review_save($order_id) {
        $selected_review_ask = get_option('wtsr_review_ask', 'no');

        if ('yes' === $selected_review_ask) {
            if ( ! empty( $_POST['ts_review_checkbox'] ) ) {
                update_post_meta( $order_id, 'Trusted Shops Review', 'yes' );
            }
        } else {
            update_post_meta( $order_id, 'Trusted Shops Review', 'yes' );
        }
    }

    public function checkbox_for_ts_display($order) {
        $is_review_allowed = get_post_meta( $order->get_id(), 'Trusted Shops Review', true );

        if (empty($is_review_allowed)) {
            $is_review_allowed = 'no';
        }

        echo '<p><strong>'.__('Trusted Shops Review').':</strong> ' . $is_review_allowed . '</p>';
    }

    /**
     * Render settings page.
     *
     * @since    0.0.1
     */
    public function save_settings() {
        if (!empty($_POST['wtsr_settings_save'])) {
            $tab = !empty($_GET['tab']) ? sanitize_text_field( $_GET['tab'] ) : 'settings';

            // Save general settings
            if (!empty($_POST['wtsr_settings_general'])) {
                $credential_ts_changed = false;

                $old_credential_ts_id = get_option('wtsr_ts_id', '');
                $old_credential_ts_email = get_option('wtsr_ts_email', '');
                $old_credential_ts_password = get_option('wtsr_ts_password', '');

                if (!empty($_POST['wtsr_ts_id'])) {
                    $wtsr_ts_id = sanitize_text_field( $_POST['wtsr_ts_id'] );

                    if (!empty($wtsr_ts_id)) {
                        delete_option( 'wtsr_ts_id' );
                        add_option( 'wtsr_ts_id', $wtsr_ts_id );

                        if ($wtsr_ts_id !== $old_credential_ts_id) {
                            $credential_ts_changed = true;
                        }
                    }
                } else {
                    delete_option( 'wtsr_ts_id' );

                    if (!empty($old_credential_ts_id)) {
                        $credential_ts_changed = true;
                    }
                }


                if (!empty($_POST['wtsr_ts_email'])) {
                    $wtsr_ts_email = sanitize_email( $_POST['wtsr_ts_email'] );

                    if (!empty($wtsr_ts_email)) {
                        delete_option( 'wtsr_ts_email' );
                        add_option( 'wtsr_ts_email', $wtsr_ts_email );

                        if ($wtsr_ts_email !== $old_credential_ts_email) {
                            $credential_ts_changed = true;
                        }
                    }
                } else {
                    delete_option( 'wtsr_ts_email' );

                    if (!empty($old_credential_ts_email)) {
                        $credential_ts_changed = true;
                    }
                }


                if (!empty($_POST['wtsr_ts_password'])) {
                    $wtsr_ts_password = sanitize_text_field( $_POST['wtsr_ts_password'] );

                    if (!empty($wtsr_ts_password)) {
                        delete_option( 'wtsr_ts_password' );
                        add_option( 'wtsr_ts_password', $wtsr_ts_password );

                        if ($wtsr_ts_password !== $old_credential_ts_password) {
                            $credential_ts_changed = true;
                        }
                    }
                } else {
                    delete_option( 'wtsr_ts_password' );

                    if (!empty($old_credential_ts_password)) {
                        $credential_ts_changed = true;
                    }
                }

                $credential_ts_changed = true;

                if ($credential_ts_changed) {
                    delete_transient('wtsr_ts_credential_empty_set');
                    delete_transient('wtsr_ts_domain_credentials_set');
                    delete_transient('wtsr_ts_credential_set');

                    delete_transient('wtsr_check_ts_credentials_empty');
                    delete_transient('wtsr_check_ts_credentials');
                    delete_transient('wtsr_check_ts_domain_credentials');

                    delete_transient('wtsr_dismiss_check_ts_domain_credentials_warning');
                    delete_transient('wtsr_dismiss_check_ts_woocommerce_only_warning');
                }

                if (!empty($_POST['wtsr_order_status'])) {
                    $wtsr_order_status = sanitize_text_field( $_POST['wtsr_order_status'] );

                    if (!empty($wtsr_order_status)) {
                        delete_option( 'wtsr_order_status' );
                        add_option( 'wtsr_order_status', $wtsr_order_status );
                    }
                }

                if (!empty($_POST['wtsr_review_variations'])) {
                    $wtsr_review_variations = sanitize_text_field( $_POST['wtsr_review_variations'] );

                    if (!empty($wtsr_review_variations)) {
                        delete_option( 'wtsr_review_variations' );
                        add_option( 'wtsr_review_variations', $wtsr_review_variations );
                    }
                }

                if (!empty($_POST['wtsr_review_ask'])) {
                    $wtsr_review_ask = sanitize_text_field( $_POST['wtsr_review_ask'] );

                    if (!empty($wtsr_review_ask)) {
                        delete_option( 'wtsr_review_ask' );
                        add_option( 'wtsr_review_ask', $wtsr_review_ask );
                    }
                }

                if (!empty($_POST['wtsr_review_ask_template_editor'])) {
                    $wtsr_review_ask_template_editor = $_POST['wtsr_review_ask_template_editor'];

                    if (!empty($wtsr_review_ask_template_editor)) {
                        delete_option( 'wtsr_review_ask_template_editor' );
                        add_option( 'wtsr_review_ask_template_editor', wp_kses_post( $wtsr_review_ask_template_editor ) );
                    }
                } else {
                    delete_option( 'wtsr_review_ask_template_editor' );
                }

                if (!empty($_POST['wtsr_initial_review_period'])) {
                    $wtsr_initial_review_period = sanitize_text_field( $_POST['wtsr_initial_review_period'] );

                    if (!empty($wtsr_initial_review_period)) {
                        delete_option( 'wtsr_initial_review_period' );
                        add_option( 'wtsr_initial_review_period', $wtsr_initial_review_period );
                    }
                }

                if (!empty($_POST['wtsr_filter_email_domain'])) {
                    $wtsr_filter_email_domain = sanitize_textarea_field( $_POST['wtsr_filter_email_domain'] );

                    if (!empty($wtsr_filter_email_domain)) {
                        $wtsr_filter_email_domain = explode("\r\n", $wtsr_filter_email_domain);

                        delete_option( 'wtsr_filter_email_domain' );
                        add_option( 'wtsr_filter_email_domain', $wtsr_filter_email_domain );
                    }
                } else {
                    delete_option( 'wtsr_filter_email_domain' );
                }

                if (!empty($_POST['wtsr_review_period'])) {
                    $wtsr_review_period = sanitize_text_field( $_POST['wtsr_review_period'] );

                    if (!empty($wtsr_review_period)) {
                        delete_option( 'wtsr_review_period' );
                        add_option( 'wtsr_review_period', $wtsr_review_period );
                    }
                }

                if (!empty($_POST['wtsr_review_approved'])) {
                    delete_option( 'wtsr_review_approved' );
                    add_option( 'wtsr_review_approved', 'yes' );
                } else {
                    delete_option( 'wtsr_review_approved' );
                    add_option( 'wtsr_review_approved', 'no' );
                }
            }

            // Save email settings
            if (!empty($_POST['wtsr_settings_email'])) {
                if (!empty($_POST['wtsr_email_template'])) {
                    $wtsr_email_template = $_POST['wtsr_email_template'];
                    $wtsr_email_template_html = wpautop($wtsr_email_template);

                    delete_option( 'wtsr_email_template' );
                    add_option( 'wtsr_email_template', wp_kses_post( $_POST['wtsr_email_template'] ) );
                } else {
                    delete_option( 'wtsr_email_template' );
                }

                $wtsr_button_colors = TSManager::get_button_colors();

                if (!empty($_POST['wtsr_button_bg_color'])) {
                    delete_option( 'wtsr_button_bg_color' );
                    add_option( 'wtsr_button_bg_color', sanitize_text_field( $_POST['wtsr_button_bg_color'] ) );
                } else {
                    delete_option( 'wtsr_button_bg_color' );
                    add_option( 'wtsr_button_bg_color', $wtsr_button_colors['bg_color'] );
                }

                if (!empty($_POST['wtsr_button_text_color'])) {
                    delete_option( 'wtsr_button_text_color' );
                    add_option( 'wtsr_button_text_color', sanitize_text_field( $_POST['wtsr_button_text_color'] ) );
                } else {
                    delete_option( 'wtsr_button_text_color' );
                    add_option( 'wtsr_button_text_color', $wtsr_button_colors['text_color'] );
                }

                if (!empty($_POST['wtsr_image_size'])) {
                    delete_option( 'wtsr_image_size' );
                    add_option( 'wtsr_image_size', sanitize_text_field( $_POST['wtsr_image_size'] ) );
                } else {
                    delete_option( 'wtsr_image_size' );
                    add_option( 'wtsr_image_size', TSManager::get_default_image_size() );
                }

                if (
                    !empty($_POST['wtsr_ts_one_star_link']) &&
                    !empty($_POST['wtsr_ts_two_star_link']) &&
                    !empty($_POST['wtsr_ts_three_star_link']) &&
                    !empty($_POST['wtsr_ts_four_star_link']) &&
                    !empty($_POST['wtsr_ts_five_star_link'])
                ) {
                    $wtsr_rating_links = array(
                        'one_star' => sanitize_text_field( $_POST['wtsr_ts_one_star_link'] ),
                        'two_star' => sanitize_text_field( $_POST['wtsr_ts_two_star_link'] ),
                        'three_star' => sanitize_text_field( $_POST['wtsr_ts_three_star_link'] ),
                        'four_star' => sanitize_text_field( $_POST['wtsr_ts_four_star_link'] ),
                        'five_star' => sanitize_text_field( $_POST['wtsr_ts_five_star_link'] ),
                    );

                    if ( 'wtsr_custom_link' === $_POST['wtsr_ts_one_star_link'] && !empty($_POST['wtsr_ts_one_star_custom_link'])) {
                        $wtsr_rating_links['custom_link']['one_star'] = sanitize_text_field($_POST['wtsr_ts_one_star_custom_link']);
                    } else {
                        $wtsr_rating_links['custom_link']['one_star'] = '';
                    }

                    if ( 'wtsr_custom_link' === $_POST['wtsr_ts_two_star_link'] && !empty($_POST['wtsr_ts_two_star_custom_link'])) {
                        $wtsr_rating_links['custom_link']['two_star'] = sanitize_text_field($_POST['wtsr_ts_two_star_custom_link']);
                    } else {
                        $wtsr_rating_links['custom_link']['two_star'] = '';
                    }

                    if ( 'wtsr_custom_link' === $_POST['wtsr_ts_three_star_link'] && !empty($_POST['wtsr_ts_three_star_custom_link'])) {
                        $wtsr_rating_links['custom_link']['three_star'] = sanitize_text_field($_POST['wtsr_ts_three_star_custom_link']);
                    } else {
                        $wtsr_rating_links['custom_link']['three_star'] = '';
                    }

                    if ( 'wtsr_custom_link' === $_POST['wtsr_ts_four_star_link'] && !empty($_POST['wtsr_ts_four_star_custom_link'])) {
                        $wtsr_rating_links['custom_link']['four_star'] = sanitize_text_field($_POST['wtsr_ts_four_star_custom_link']);
                    } else {
                        $wtsr_rating_links['custom_link']['four_star'] = '';
                    }

                    if ( 'wtsr_custom_link' === $_POST['wtsr_ts_five_star_link'] && !empty($_POST['wtsr_ts_five_star_custom_link'])) {
                        $wtsr_rating_links['custom_link']['five_star'] = sanitize_text_field($_POST['wtsr_ts_five_star_custom_link']);
                    } else {
                        $wtsr_rating_links['custom_link']['five_star'] = '';
                    }

                    delete_option( 'wtsr_rating_links' );
                    add_option( 'wtsr_rating_links', $wtsr_rating_links );
                }
            }

            wp_redirect( admin_url( 'admin.php?page=wp2leads-wtsr&tab=' . trim($tab) ) );
        }
    }
}
